"""
Type annotations for botocore.hooks module.

Copyright 2024 Vlad Emelianov
"""

from logging import Logger
from typing import Any, NamedTuple, TypeVar

from botocore.compat import accepts_kwargs as accepts_kwargs
from botocore.utils import EVENT_ALIASES as EVENT_ALIASES

logger: Logger = ...

_R = TypeVar("_R")

class _NodeList(NamedTuple):
    first: Any
    middle: Any
    last: Any

class NodeList(_NodeList):
    def __copy__(self: _R) -> _R: ...

def first_non_none_response(responses: Any, default: Any | None = ...) -> Any: ...

class BaseEventHooks:
    def emit(self, event_name: Any, **kwargs: Any) -> Any: ...
    def register(
        self,
        event_name: str,
        handler: Any,
        unique_id: str | None = ...,
        unique_id_uses_count: bool = ...,
    ) -> None: ...
    def register_first(
        self,
        event_name: str,
        handler: Any,
        unique_id: str | None = ...,
        unique_id_uses_count: bool = ...,
    ) -> None: ...
    def register_last(
        self,
        event_name: str,
        handler: Any,
        unique_id: str | None = ...,
        unique_id_uses_count: bool = ...,
    ) -> None: ...
    def unregister(
        self,
        event_name: str,
        handler: Any | None = ...,
        unique_id: str | None = ...,
        unique_id_uses_count: bool = ...,
    ) -> None: ...

class HierarchicalEmitter(BaseEventHooks):
    def __init__(self) -> None: ...
    def emit(self, event_name: str, **kwargs: Any) -> Any: ...
    def emit_until_response(self, event_name: str, **kwargs: Any) -> Any: ...
    def unregister(
        self,
        event_name: str,
        handler: Any | None = ...,
        unique_id: str | None = ...,
        unique_id_uses_count: bool = ...,
    ) -> None: ...
    def __copy__(self: _R) -> _R: ...

class EventAliaser(BaseEventHooks):
    def __init__(self, event_emitter: BaseEventHooks, event_aliases: Any | None = ...) -> None: ...
    def emit(self, event_name: str, **kwargs: Any) -> Any: ...
    def emit_until_response(self, event_name: str, **kwargs: Any) -> Any: ...
    def register(
        self,
        event_name: str,
        handler: Any,
        unique_id: Any | None = ...,
        unique_id_uses_count: bool = ...,
    ) -> Any: ...
    def register_first(
        self,
        event_name: str,
        handler: Any,
        unique_id: str | None = ...,
        unique_id_uses_count: bool = ...,
    ) -> Any: ...
    def register_last(
        self,
        event_name: str,
        handler: Any,
        unique_id: str | None = ...,
        unique_id_uses_count: bool = ...,
    ) -> Any: ...
    def unregister(
        self,
        event_name: str,
        handler: Any | None = ...,
        unique_id: str | None = ...,
        unique_id_uses_count: bool = ...,
    ) -> Any: ...
    def __copy__(self: _R) -> _R: ...



"""
Type annotations for botocore.auth module.

Copyright 2024 Vlad Emelianov
"""

from http.client import HTTPMessage
from logging import Logger
from typing import Any, Iterable, Mapping
from urllib.parse import SplitResult

from botocore.awsrequest import AWSRequest
from botocore.compat import HAS_CRT as HAS_CRT
from botocore.compat import MD5_AVAILABLE as MD5_AVAILABLE
from botocore.credentials import Credentials, ReadOnlyCredentials
from botocore.crt.auth import CRT_AUTH_TYPE_MAPS as CRT_AUTH_TYPE_MAPS
from botocore.utils import IdentityCache

logger: Logger = ...

EMPTY_SHA256_HASH: str
PAYLOAD_BUFFER: int
ISO8601: str
SIGV4_TIMESTAMP: str
SIGNED_HEADERS_BLACKLIST: list[str]
UNSIGNED_PAYLOAD: str
STREAMING_UNSIGNED_PAYLOAD_TRAILER: str

class BaseSigner:
    REQUIRES_REGION: bool = ...
    REQUIRES_TOKEN: bool = ...
    def add_auth(self, request: AWSRequest) -> AWSRequest | None: ...

class TokenSigner(BaseSigner):
    def __init__(self, auth_token: str) -> None: ...

class SigV2Auth(BaseSigner):
    def __init__(self, credentials: Credentials | ReadOnlyCredentials) -> None:
        self.credentials: Credentials | ReadOnlyCredentials

    def calc_signature(self, request: AWSRequest, params: Mapping[str, Any]) -> tuple[str, str]: ...
    def add_auth(self, request: AWSRequest) -> AWSRequest: ...

class SigV3Auth(BaseSigner):
    def __init__(self, credentials: Credentials | ReadOnlyCredentials) -> None:
        self.credentials: Credentials | ReadOnlyCredentials

    def add_auth(self, request: AWSRequest) -> None: ...

class SigV4Auth(BaseSigner):
    REQUIRES_REGION: bool = ...
    def __init__(
        self, credentials: Credentials | ReadOnlyCredentials, service_name: str, region_name: str
    ) -> None:
        self.credentials: Credentials | ReadOnlyCredentials

    def headers_to_sign(self, request: AWSRequest) -> HTTPMessage: ...
    def canonical_query_string(self, request: AWSRequest) -> str: ...
    def canonical_headers(self, headers_to_sign: Iterable[str]) -> str: ...
    def signed_headers(self, headers_to_sign: Iterable[str]) -> str: ...
    def payload(self, request: AWSRequest) -> str: ...
    def canonical_request(self, request: AWSRequest) -> str: ...
    def scope(self, request: AWSRequest) -> str: ...
    def credential_scope(self, request: AWSRequest) -> str: ...
    def string_to_sign(self, request: AWSRequest, canonical_request: str) -> str: ...
    def signature(self, string_to_sign: str, request: AWSRequest) -> str: ...
    def add_auth(self, request: AWSRequest) -> None: ...

class S3SigV4Auth(SigV4Auth): ...

class S3ExpressAuth(S3SigV4Auth):
    REQUIRES_IDENTITY_CACHE: bool = ...

    def __init__(
        self,
        credentials: Credentials | ReadOnlyCredentials,
        service_name: str,
        region_name: str,
        *,
        identity_cache: IdentityCache,
    ) -> None: ...

class S3ExpressPostAuth(S3ExpressAuth): ...

class S3ExpressQueryAuth(S3ExpressAuth):
    DEFAULT_EXPIRES: int = ...

    def __init__(
        self,
        credentials: Credentials | ReadOnlyCredentials,
        service_name: str,
        region_name: str,
        *,
        identity_cache: IdentityCache,
        expires: int = ...,
    ) -> None: ...

class SigV4QueryAuth(SigV4Auth):
    DEFAULT_EXPIRES: int = ...
    def __init__(
        self,
        credentials: Credentials | ReadOnlyCredentials,
        service_name: str,
        region_name: str,
        expires: int = ...,
    ) -> None: ...

class S3SigV4QueryAuth(SigV4QueryAuth):
    def payload(self, request: AWSRequest) -> str: ...

class S3SigV4PostAuth(SigV4Auth):
    def add_auth(self, request: AWSRequest) -> None: ...

class HmacV1Auth(BaseSigner):
    QSAOfInterest: list[str] = ...
    def __init__(
        self,
        credentials: Credentials | ReadOnlyCredentials,
        service_name: str | None = ...,
        region_name: str | None = ...,
    ) -> None:
        self.credentials: Credentials | ReadOnlyCredentials

    def sign_string(self, string_to_sign: str) -> str: ...
    def canonical_standard_headers(self, headers: Mapping[str, Any]) -> str: ...
    def canonical_custom_headers(self, headers: Mapping[str, Any]) -> str: ...
    def unquote_v(self, nv: str) -> tuple[str, str] | str: ...
    def canonical_resource(self, split: SplitResult, auth_path: str | None = ...) -> str: ...
    def canonical_string(
        self,
        method: str,
        split: SplitResult,
        headers: Mapping[str, Any],
        expires: int | None = ...,
        auth_path: str | None = ...,
    ) -> Any: ...
    def get_signature(
        self,
        method: str,
        split: SplitResult,
        headers: Mapping[str, Any],
        expires: int | None = ...,
        auth_path: str | None = ...,
    ) -> Any: ...
    def add_auth(self, request: AWSRequest) -> None: ...

class HmacV1QueryAuth(HmacV1Auth):
    DEFAULT_EXPIRES: int = ...
    def __init__(self, credentials: Credentials | ReadOnlyCredentials, expires: int = ...) -> None:
        self.credentials: Credentials | ReadOnlyCredentials

class HmacV1PostAuth(HmacV1Auth):
    def add_auth(self, request: AWSRequest) -> None: ...

class BearerAuth(TokenSigner):
    def add_auth(self, request: AWSRequest) -> None: ...

def resolve_auth_type(auth_trait: Iterable[str]) -> str: ...

AUTH_TYPE_MAPS: dict[str, type[BaseSigner]]
AUTH_TYPE_TO_SIGNATURE_VERSION: dict[str, str]



"""
Type annotations for botocore.eventstream module.

Copyright 2024 Vlad Emelianov
"""

from typing import Any, Generic, Iterator, TypeVar

from botocore.exceptions import EventStreamError as EventStreamError
from botocore.model import StructureShape
from botocore.parsers import BaseEventStreamParser

_T = TypeVar("_T")

class ParserError(Exception): ...

class DuplicateHeader(ParserError):
    def __init__(self, header: str) -> None: ...

class InvalidHeadersLength(ParserError):
    def __init__(self, length: int) -> None: ...

class InvalidPayloadLength(ParserError):
    def __init__(self, length: int) -> None: ...

class ChecksumMismatch(ParserError):
    def __init__(self, expected: int, calculated: int) -> None: ...

class NoInitialResponseError(ParserError):
    def __init__(self) -> None: ...

class DecodeUtils:
    UINT8_BYTE_FORMAT: str = ...
    UINT16_BYTE_FORMAT: str = ...
    UINT32_BYTE_FORMAT: str = ...
    INT8_BYTE_FORMAT: str = ...
    INT16_BYTE_FORMAT: str = ...
    INT32_BYTE_FORMAT: str = ...
    INT64_BYTE_FORMAT: str = ...
    PRELUDE_BYTE_FORMAT: str = ...
    UINT_BYTE_FORMAT: dict[int, str] = ...
    @staticmethod
    def unpack_true(data: bytes) -> tuple[bool, int]: ...
    @staticmethod
    def unpack_false(data: bytes) -> tuple[bool, int]: ...
    @staticmethod
    def unpack_uint8(data: bytes) -> tuple[int, int]: ...
    @staticmethod
    def unpack_uint32(data: bytes) -> tuple[int, int]: ...
    @staticmethod
    def unpack_int8(data: bytes) -> tuple[int, int]: ...
    @staticmethod
    def unpack_int16(data: bytes) -> tuple[int, int]: ...
    @staticmethod
    def unpack_int32(data: bytes) -> tuple[int, int]: ...
    @staticmethod
    def unpack_int64(data: bytes) -> tuple[int, int]: ...
    @staticmethod
    def unpack_byte_array(data: bytes, length_byte_size: int = ...) -> tuple[bytes, int]: ...
    @staticmethod
    def unpack_utf8_string(data: bytes, length_byte_size: int = ...) -> tuple[str, int]: ...
    @staticmethod
    def unpack_uuid(data: bytes) -> tuple[bytes, int]: ...
    @staticmethod
    def unpack_prelude(data: bytes) -> tuple[tuple[int, int, int], int]: ...

class MessagePrelude:
    def __init__(self, total_length: int, headers_length: int, crc: int) -> None:
        self.total_length: int = ...
        self.headers_length: int = ...
        self.crc: int = ...

    @property
    def payload_length(self) -> int: ...
    @property
    def payload_end(self) -> int: ...
    @property
    def headers_end(self) -> int: ...

class EventStreamMessage:
    def __init__(self, prelude: int, headers: int, payload: bytes, crc: int) -> None:
        self.prelude: int = ...
        self.headers: int = ...
        self.payload: bytes = ...
        self.crc: int = ...

    def to_response_dict(self, status_code: int = ...) -> dict[str, Any]: ...

class EventStreamHeaderParser:
    def __init__(self) -> None: ...
    def parse(self, data: bytes) -> dict[str, Any]: ...

class EventStreamBuffer:
    def __init__(self) -> None: ...
    def add_data(self, data: bytes) -> None: ...
    def next(self) -> EventStreamMessage: ...
    def __next__(self) -> EventStreamMessage: ...
    def __iter__(self: _T) -> _T: ...

class EventStream(Generic[_T]):
    def __init__(
        self,
        raw_stream: Any,
        output_shape: StructureShape,
        parser: BaseEventStreamParser,
        operation_name: str,
    ) -> None: ...
    def __iter__(self) -> Iterator[_T]: ...
    def get_initial_response(self) -> _T: ...
    def close(self) -> None: ...



"""
Type annotations for botocore.retries.standard module.

Copyright 2024 Vlad Emelianov
"""

from logging import Logger
from typing import Any, Callable, Sequence

from botocore.client import BaseClient
from botocore.exceptions import BotoCoreError
from botocore.exceptions import ConnectionError as ConnectionError
from botocore.exceptions import ConnectTimeoutError as ConnectTimeoutError
from botocore.exceptions import HTTPClientError as HTTPClientError
from botocore.exceptions import ReadTimeoutError as ReadTimeoutError
from botocore.model import OperationModel
from botocore.retries import quota as quota
from botocore.retries import special as special
from botocore.retries.base import BaseRetryableChecker as BaseRetryableChecker
from botocore.retries.base import BaseRetryBackoff as BaseRetryBackoff
from botocore.retries.quota import RetryQuota

logger: Logger = ...

DEFAULT_MAX_ATTEMPTS: int = ...

def register_retry_handler(client: BaseClient, max_attempts: int = ...) -> RetryHandler: ...

class RetryHandler:
    def __init__(
        self,
        retry_policy: RetryPolicy,
        retry_event_adapter: RetryEventAdapter,
        retry_quota: RetryQuota,
    ) -> None: ...
    def needs_retry(self, **kwargs: Any) -> float | None: ...

class RetryEventAdapter:
    def create_retry_context(self, **kwargs: Any) -> Any: ...
    def adapt_retry_response_from_context(self, context: Any) -> None: ...

class RetryContext:
    def __init__(
        self,
        attempt_number: int,
        operation_model: OperationModel | None = ...,
        parsed_response: Any = ...,
        http_response: Any = ...,
        caught_exception: Exception | None = ...,
        request_context: Any = ...,
    ) -> None:
        self.attempt_number: int = ...
        self.operation_model: OperationModel | None = ...
        self.parsed_response: Any = ...
        self.http_response: Any = ...
        self.caught_exception: Exception | None = ...
        self.request_context: Any = ...

    def get_error_code(self) -> int | None: ...
    def add_retry_metadata(self, **kwargs: Any) -> None: ...
    def get_retry_metadata(self) -> Any: ...

class RetryPolicy:
    def __init__(
        self, retry_checker: BaseRetryableChecker, retry_backoff: BaseRetryBackoff
    ) -> None: ...
    def should_retry(self, context: Any) -> Any: ...
    def compute_retry_delay(self, context: Any) -> float: ...

class ExponentialBackoff(BaseRetryBackoff):
    def __init__(self, max_backoff: int = ..., random: Callable[[], float] = ...) -> None: ...
    def delay_amount(self, context: Any) -> float: ...

class MaxAttemptsChecker(BaseRetryableChecker):
    def __init__(self, max_attempts: int) -> None: ...
    def is_retryable(self, context: Any) -> bool: ...

class TransientRetryableChecker(BaseRetryableChecker):
    def __init__(
        self,
        transient_error_codes: Sequence[str] | None = ...,
        transient_status_codes: Sequence[int] | None = ...,
        transient_exception_cls: Sequence[type[BotoCoreError]] | None = ...,
    ) -> None: ...
    def is_retryable(self, context: Any) -> bool: ...

class ThrottledRetryableChecker(BaseRetryableChecker):
    def __init__(self, throttled_error_codes: Sequence[str] | None = ...) -> None: ...
    def is_retryable(self, context: Any) -> bool: ...

class ModeledRetryableChecker(BaseRetryableChecker):
    def __init__(self) -> None: ...
    def is_retryable(self, context: Any) -> Any: ...

class ModeledRetryErrorDetector:
    TRANSIENT_ERROR: str = ...
    THROTTLING_ERROR: str = ...
    def detect_error_type(self, context: Any) -> Any: ...

class ThrottlingErrorDetector:
    def __init__(self, retry_event_adapter: Any) -> None: ...
    def is_throttling_error(self, **kwargs: Any) -> Any: ...

class StandardRetryConditions(BaseRetryableChecker):
    def __init__(self, max_attempts: Any = ...) -> None: ...
    def is_retryable(self, context: Any) -> Any: ...

class OrRetryChecker(BaseRetryableChecker):
    def __init__(self, checkers: Any) -> None: ...
    def is_retryable(self, context: Any) -> Any: ...

class RetryQuotaChecker:
    def __init__(self, quota: Any) -> None: ...  # noqa: F811
    def acquire_retry_quota(self, context: Any) -> bool: ...
    def release_retry_quota(self, context: Any, http_response: Any, **kwargs: Any) -> None: ...



"""
Type annotations for botocore.configloader module.

Copyright 2024 Vlad Emelianov
"""

from typing import Any

def multi_file_load_config(*filenames: str) -> dict[str, Any]: ...
def load_config(config_filename: str) -> dict[str, Any]: ...
def raw_config_parse(config_filename: str, parse_subsections: bool = ...) -> dict[str, Any]: ...
def build_profile_map(parsed_ini_config: dict[str, Any]) -> dict[str, Any]: ...



"""
Type annotations for botocore.retries.quota module.

Copyright 2024 Vlad Emelianov
"""

from typing import Any

class RetryQuota:
    INITIAL_CAPACITY: int = ...
    def __init__(self, initial_capacity: Any = ..., lock: Any | None = ...) -> None: ...
    def acquire(self, capacity_amount: Any) -> Any: ...
    def release(self, capacity_amount: Any) -> None: ...
    @property
    def available_capacity(self) -> Any: ...



"""
Type annotations for botocore.docs.method module.

Copyright 2024 Vlad Emelianov
"""

from typing import Any, Sequence

from botocore.hooks import BaseEventHooks
from botocore.model import OperationModel

AWS_DOC_BASE: str = ...

def get_instance_public_methods(instance: Any) -> dict[str, Any]: ...
def document_model_driven_signature(
    section: Any,
    name: str,
    operation_model: OperationModel,
    include: Sequence[str] | None = ...,
    exclude: Sequence[str] | None = ...,
) -> None: ...
def document_custom_signature(
    section: Any,
    name: str,
    method: Any,
    include: Sequence[str] | None = ...,
    exclude: Sequence[str] | None = ...,
) -> None: ...
def document_custom_method(section: Any, method_name: str, method: Any) -> None: ...
def document_model_driven_method(
    section: Any,
    method_name: str,
    operation_model: OperationModel,
    event_emitter: BaseEventHooks,
    method_description: str | None = ...,
    example_prefix: str | None = ...,
    include_input: dict[str, Any] | None = ...,
    include_output: dict[str, Any] | None = ...,
    exclude_input: Sequence[str] | None = ...,
    exclude_output: Sequence[str] | None = ...,
    document_output: bool = ...,
    include_signature: bool = ...,
) -> None: ...



"""
Type annotations for botocore.loaders module.

Copyright 2024 Vlad Emelianov
"""

from logging import Logger
from typing import Any, Callable, Iterable, Mapping

from botocore.compat import OrderedDict as OrderedDict
from botocore.exceptions import DataNotFoundError as DataNotFoundError
from botocore.exceptions import UnknownServiceError as UnknownServiceError
from botocore.utils import deep_merge as deep_merge

logger: Logger = ...

HAS_GZIP: bool

def instance_cache(func: Callable[..., Any]) -> Callable[..., Any]: ...

class JSONFileLoader:
    def exists(self, file_path: str) -> bool: ...
    def load_file(self, file_path: str) -> Any: ...

def create_loader(search_path_string: str | None = ...) -> Loader: ...

class Loader:
    FILE_LOADER_CLASS: type[JSONFileLoader] = ...
    BUILTIN_DATA_PATH: str = ...
    CUSTOMER_DATA_PATH: str = ...
    BUILTIN_EXTRAS_TYPES: list[str] = ...
    def __init__(
        self,
        extra_search_paths: Iterable[str] | None = ...,
        file_loader: JSONFileLoader | None = ...,
        cache: Any | None = ...,
        include_default_search_paths: bool = ...,
        include_default_extras: bool = ...,
    ) -> None:
        self.file_loader: JSONFileLoader

    @property
    def search_paths(self) -> list[str]: ...
    @property
    def extras_types(self) -> list[str]: ...
    def list_available_services(self, type_name: str) -> list[str]: ...
    def determine_latest_version(self, service_name: str, type_name: str) -> str: ...
    def list_api_versions(self, service_name: str, type_name: str) -> list[str]: ...
    def load_data_with_path(self, name: str) -> tuple[dict[str, Any], str]: ...
    def load_service_model(
        self, service_name: str, type_name: str, api_version: Any | None = ...
    ) -> Any: ...
    def load_data(self, name: str) -> Any: ...
    def is_builtin_path(self, path: str) -> bool: ...

class ExtrasProcessor:
    def process(
        self, original_model: Mapping[str, Any], extra_models: Iterable[Mapping[str, Any]]
    ) -> None: ...



"""
Type annotations for botocore.retries.base module.

Copyright 2024 Vlad Emelianov
"""

from typing import Any

class BaseRetryBackoff:
    def delay_amount(self, context: Any) -> float: ...

class BaseRetryableChecker:
    def is_retryable(self, context: Any) -> bool: ...



"""
Type annotations for botocore.docs.bcdoc.style module.

Copyright 2024 Vlad Emelianov
"""

from logging import Logger
from typing import Any

from botocore.docs.bcdoc.restdoc import ReSTDocument

logger: Logger = ...
PUNCTUATION_CHARACTERS: tuple[str, ...] = ...

class BaseStyle:
    def __init__(self, doc: ReSTDocument, indent_width: int = ...) -> None: ...
    @property
    def indentation(self) -> int: ...
    @indentation.setter
    def indentation(self, value: int) -> None: ...
    def new_paragraph(self) -> str: ...
    def indent(self) -> None: ...
    def dedent(self) -> None: ...
    def spaces(self) -> str: ...
    def bold(self, s: str) -> str: ...
    def ref(self, link: str | None, title: str | None = ...) -> str: ...
    def h2(self, s: str) -> str: ...
    def h3(self, s: str) -> str: ...
    def underline(self, s: str) -> str: ...
    def italics(self, s: str) -> str: ...
    def add_trailing_space_to_previous_write(self) -> None: ...

class ReSTStyle(BaseStyle):
    def __init__(self, doc: ReSTDocument, indent_width: int = ...) -> None: ...
    def new_paragraph(self) -> None: ...  # type: ignore[override]
    def new_line(self) -> None: ...
    def start_bold(self, attrs: Any = ...) -> None: ...
    def end_bold(self) -> None: ...
    def start_b(self, attrs: Any = ...) -> None: ...
    def end_b(self) -> None: ...
    def bold(self, s: str) -> None: ...  # type: ignore[override]
    def ref(self, title: str, link: str | None = ...) -> None: ...  # type: ignore[override]
    def h1(self, s: str) -> None: ...
    def h2(self, s: str) -> None: ...  # type: ignore[override]
    def h3(self, s: str) -> None: ...  # type: ignore[override]
    def start_italics(self, attrs: Any = ...) -> None: ...
    def end_italics(self) -> None: ...
    def italics(self, s: str) -> None: ...  # type: ignore[override]
    def start_p(self, attrs: Any = ...) -> None: ...
    def end_p(self) -> None: ...
    def start_code(self, attrs: Any = ...) -> None: ...
    def end_code(self) -> None: ...
    def code(self, s: str) -> None: ...
    def start_note(self, attrs: Any = ...) -> None: ...
    def end_note(self) -> None: ...
    def start_important(self, attrs: Any = ...) -> None: ...
    def end_important(self) -> None: ...
    def start_danger(self, attrs: Any = ...) -> None: ...
    def end_danger(self) -> None: ...
    def start_a(self, attrs: Any = ...) -> None: ...
    def link_target_definition(self, refname: str, link: str) -> None: ...
    def sphinx_reference_label(self, label: str, text: str | None = ...) -> None: ...
    def end_a(self, next_child: Any = ...) -> None: ...
    def start_i(self, attrs: Any = ...) -> None: ...
    def end_i(self) -> None: ...
    def start_li(self, attrs: Any = ...) -> None: ...
    def end_li(self) -> None: ...
    def li(self, s: str) -> None: ...
    def start_ul(self, attrs: Any = ...) -> None: ...
    def end_ul(self) -> None: ...
    def start_ol(self, attrs: Any = ...) -> None: ...
    def end_ol(self) -> None: ...
    def start_examples(self, attrs: Any = ...) -> None: ...
    def end_examples(self) -> None: ...
    def start_fullname(self, attrs: Any = ...) -> None: ...
    def end_fullname(self) -> None: ...
    def start_codeblock(self, attrs: Any = ...) -> None: ...
    def end_codeblock(self) -> None: ...
    def codeblock(self, code: str) -> None: ...
    def toctree(self) -> None: ...
    def tocitem(self, item: str, file_name: str | None = ...) -> None: ...
    def hidden_toctree(self) -> None: ...
    def hidden_tocitem(self, item: str) -> None: ...
    def table_of_contents(self, title: str | None = ..., depth: int | None = ...) -> None: ...
    def start_sphinx_py_class(self, class_name: str) -> None: ...
    def end_sphinx_py_class(self) -> None: ...
    def start_sphinx_py_method(self, method_name: str, parameters: Any = ...) -> None: ...
    def end_sphinx_py_method(self) -> None: ...
    def start_sphinx_py_attr(self, attr_name: str) -> None: ...
    def end_sphinx_py_attr(self) -> None: ...
    def write_py_doc_string(self, docstring: str) -> None: ...
    def external_link(self, title: str, link: str) -> None: ...
    def internal_link(self, title: str, page: str) -> None: ...



"""
Type annotations for botocore.docs.waiter module.

Copyright 2024 Vlad Emelianov
"""

from typing import Any

from botocore.client import BaseClient
from botocore.hooks import BaseEventHooks
from botocore.model import ServiceModel
from botocore.waiter import WaiterModel

class WaiterDocumenter:
    def __init__(
        self, client: BaseClient, service_waiter_model: WaiterModel, root_docs_path: str
    ) -> None: ...
    def document_waiters(self, section: Any) -> None: ...

def document_wait_method(
    section: Any,
    waiter_name: str,
    event_emitter: BaseEventHooks,
    service_model: ServiceModel,
    service_waiter_model: WaiterModel,
    include_signature: bool = ...,
) -> None: ...



"""
Type annotations for botocore.docs.example module.

Copyright 2024 Vlad Emelianov
"""

from typing import Any

from botocore.docs.shape import ShapeDocumenter
from botocore.model import Shape

class BaseExampleDocumenter(ShapeDocumenter):
    def document_example(
        self,
        section: Any,
        shape: Shape,
        prefix: str | None = ...,
        include: list[str] | None = ...,
        exclude: list[str] | None = ...,
    ) -> None: ...
    def document_recursive_shape(self, section: Any, shape: Shape, **kwargs: Any) -> None: ...
    def document_shape_default(
        self,
        section: Any,
        shape: Shape,
        history: Any,
        include: list[str] | None = ...,
        exclude: list[str] | None = ...,
        **kwargs: Any,
    ) -> None: ...
    def document_shape_type_string(
        self,
        section: Any,
        shape: Shape,
        history: Any,
        include: list[str] | None = ...,
        exclude: list[str] | None = ...,
        **kwargs: Any,
    ) -> None: ...
    def document_shape_type_list(
        self,
        section: Any,
        shape: Shape,
        history: Any,
        include: list[str] | None = ...,
        exclude: list[str] | None = ...,
        **kwargs: Any,
    ) -> None: ...
    def document_shape_type_structure(
        self,
        section: Any,
        shape: Shape,
        history: Any,
        include: list[str] | None = ...,
        exclude: list[str] | None = ...,
        **kwargs: Any,
    ) -> None: ...
    def document_shape_type_map(
        self,
        section: Any,
        shape: Shape,
        history: Any,
        include: list[str] | None = ...,
        exclude: list[str] | None = ...,
        **kwargs: Any,
    ) -> None: ...

class ResponseExampleDocumenter(BaseExampleDocumenter):
    EVENT_NAME: str = ...
    def document_shape_type_event_stream(
        self,
        section: Any,
        shape: Shape,
        history: Any,
        **kwargs: Any,
    ) -> None: ...

class RequestExampleDocumenter(BaseExampleDocumenter):
    EVENT_NAME: str = ...
    def document_shape_type_structure(
        self,
        section: Any,
        shape: Shape,
        history: Any,
        include: list[str] | None = ...,
        exclude: list[str] | None = ...,
        **kwargs: Any,
    ) -> None: ...



"""
Type annotations for botocore.docs.utils module.

Copyright 2024 Vlad Emelianov
"""

from typing import Any, Callable

from botocore.model import ServiceModel

def py_type_name(type_name: str) -> str: ...
def py_default(type_name: str) -> str: ...
def get_official_service_name(service_model: ServiceModel) -> str: ...

_DocumentedShape: Any = ...

class DocumentedShape(_DocumentedShape):
    def __new__(
        cls,
        name: str,
        type_name: str,
        documentation: Any,
        metadata: Any = ...,
        members: Any = ...,
        required_members: Any = ...,
    ) -> Any: ...

class AutoPopulatedParam:
    def __init__(self, name: str, param_description: str | None = ...) -> None: ...
    def document_auto_populated_param(
        self, event_name: str, section: Any, **kwargs: Any
    ) -> None: ...

class HideParamFromOperations:
    def __init__(
        self, service_name: str, parameter_name: str, operation_names: list[str]
    ) -> None: ...
    def hide_param(self, event_name: str, section: Any, **kwargs: Any) -> None: ...

class AppendParamDocumentation:
    def __init__(self, parameter_name: str, doc_string: str) -> None: ...
    def append_documentation(self, event_name: str, section: Any, **kwargs: Any) -> None: ...

_CONTROLS: dict[str, str] = ...
_ESCAPE_CONTROLS_RE: Any = ...
_CONTROLS_MATCH_HANDLER: Callable[..., str] = ...

def escape_controls(value: str) -> str: ...



"""
Type annotations for botocore.docs.paginator module.

Copyright 2024 Vlad Emelianov
"""

from typing import Any

from botocore.client import BaseClient
from botocore.hooks import BaseEventHooks
from botocore.model import ServiceModel

class PaginatorDocumenter:
    def __init__(
        self, client: BaseClient, service_paginator_model: ServiceModel, root_docs_path: str
    ) -> None: ...
    def document_paginators(self, section: Any) -> None: ...

def document_paginate_method(
    section: Any,
    paginator_name: str,
    event_emitter: BaseEventHooks,
    service_model: ServiceModel,
    paginator_config: Any,
    include_signature: bool = ...,
) -> None: ...



"""
Type annotations for botocore.translate module.

Copyright 2024 Vlad Emelianov
"""

from typing import Any

from botocore.utils import merge_dicts as merge_dicts

def build_retry_config(
    endpoint_prefix: str,
    retry_model: Any,
    definitions: Any,
    client_retry_config: Any | None = ...,
) -> Any: ...
def resolve_references(config: Any, definitions: Any) -> None: ...



"""
Type annotations for botocore.history module.

Copyright 2024 Vlad Emelianov
"""

from logging import Logger
from typing import Any

logger: Logger = ...

HISTORY_RECORDER: HistoryRecorder | None

class BaseHistoryHandler:
    def emit(self, event_type: str, payload: Any, source: str) -> None: ...

class HistoryRecorder:
    def __init__(self) -> None: ...
    def enable(self) -> None: ...
    def disable(self) -> None: ...
    def add_handler(self, handler: BaseHistoryHandler) -> None: ...
    def record(self, event_type: str, payload: Any, source: str = ...) -> None: ...

def get_global_history_recorder() -> HistoryRecorder: ...
